--*******************************************************************************************************************************
--
-- Description: This module performs read and write operations to the two QDR II+ SRAM chips using a modified Xilinx MIG core.
--					
-- Rev History: 10-25-2013 Initial Release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 
use work.model_specific_package.all;
use work.transceiver_package.all;

entity dual_qdr_sram is 
	port (
	--  Reset and clocks connected to the application logic.
	rst         				: in std_logic;						-- Active high asynchronous reset.
	clk200 						: in std_logic;						-- 200 MHz clock on global clock buffer.
	sram_clk 					: out std_logic;					-- Half frequency SRAM clock generated by the core.
	
	-- QDR II+ SRAM-A controller interface signals.
	sram_a_wr_cmd				: in std_logic;						-- Single period write command strobe.
	sram_a_wr_addr				: in std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for write operation.
	sram_a_wr_data				: in std_logic_vector(71 downto 0);	-- Data bus carrying four 18-bit QDR II+ SRAM write data values.
	sram_a_wr_bw_n				: in std_logic_vector(7 downto 0);	-- Eight active low byte enables assigned to the QDR II+ SRAM write data values.
	sram_a_rd_cmd				: in std_logic;						-- Single period read command strobe.
	sram_a_rd_addr				: in std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for read operation.
	sram_a_rd_data				: out std_logic_vector(71 downto 0);-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	sram_a_rd_valid				: out std_logic;					-- Logic one indicates that read data is available from the QDR II+ SRAM.
	sram_a_calib_complete		: out std_logic;					-- Logic one indicates that the controller has completed timing calibration.
	sram_a_rst_clk         		: out std_logic;					-- Active high reset synchronized to the half frequency SRAM clock.
	
	-- QDR II+ SRAM-B controller interface signals.
	sram_b_wr_cmd				: in std_logic;						-- Single period write command strobe.
	sram_b_wr_addr				: in std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for write operation.
	sram_b_wr_data				: in std_logic_vector(71 downto 0);	-- Data bus carrying four 18-bit QDR II+ SRAM write data values.
	sram_b_wr_bw_n				: in std_logic_vector(7 downto 0);	-- Eight active low byte enables assigned to the QDR II+ SRAM write data values.
	sram_b_rd_cmd				: in std_logic;						-- Single period read command strobe.
	sram_b_rd_addr				: in std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for read operation.
	sram_b_rd_data				: out std_logic_vector(71 downto 0);-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	sram_b_rd_valid				: out std_logic;					-- Logic one indicates that read data is available from the QDR II+ SRAM.
	sram_b_calib_complete		: out std_logic;					-- Logic one indicates that the controller has completed timing calibration.
	sram_b_rst_clk         		: out std_logic;					-- Active high reset synchronized to the half frequency SRAM clock.
	
	--	The following signals connect to the QDR II+ SRAM devices through FPGA pins.
	qdriip_sys_clk_p 			: in std_logic;						-- SRAM clock from external oscillator.
	qdriip_sys_clk_n 			: in std_logic;						-- Differential pair (see above).
	qdriip_cq					: in DUAL_1;						-- QDR II+ SRAM echo clock.
	qdriip_q					: in DUAL_18;						-- QDR II+ SRAM synchronous data outputs.
	qdriip_qvld					: in DUAL_1;						-- QDR II+ SRAM read data valid flag.
	qdriip_k_p					: out DUAL_1;						-- QDR II+ SRAM input clock.
	qdriip_k_n					: out DUAL_1;						-- QDR II+ SRAM differential pair (see above).
	qdriip_d					: out DUAL_18;						-- QDR II+ SRAM synchronous data inputs.
	qdriip_sa					: out DUAL_QDR;						-- QDR II+ SRAM synchronous address inputs.
	qdriip_w_n					: out DUAL_1;						-- QDR II+ SRAM synchronous active low write enable.
	qdriip_r_n					: out DUAL_1;						-- QDR II+ SRAM synchronous active low read enable.
	qdriip_bw_n					: out DUAL_2;						-- QDR II+ SRAM synchronous byte enables.
	qdriip_dll_off_n			: out DUAL_1						-- QDR II+ SRAM active low DLL disable.
	);	
end dual_qdr_sram;

architecture behavioral of dual_qdr_sram is

	signal qdriip_cq_slv		: std_logic_vector(1 downto 0);		-- Echo clock type conversion for the MIG core.
	signal qdriip_k_p_slv		: std_logic_vector(1 downto 0);		-- Write clock type conversion for the MIG core.
	signal qdriip_k_n_slv		: std_logic_vector(1 downto 0);		-- Write clock type conversion for the MIG core.
	signal sys_rst_local		: std_logic;						-- Shared system reset routed from SRAM-A to SRAM-B.
	signal clocks_local			: std_logic_vector(6 downto 0);		-- Shared MMCM/PLL clocks routed from SRAM-A to SRAM-B.
	
begin
 	
--****************************************************** SRAM-A INTERFACE ******************************************************

	-- Convert between standard logic and single bit standard logic vector types.
	qdriip_cq_slv(0) <= qdriip_cq(0);
	qdriip_k_p(0) <= qdriip_k_p_slv(0);
	qdriip_k_n(0) <= qdriip_k_n_slv(0);
	
	SRAM_A_MIG : entity work.qdr_sram_a
	port map (
		sys_rst_out				=> sys_rst_local,
		clocks_out				=> clocks_local,
		sys_clk_p				=> qdriip_sys_clk_p,
		sys_clk_n				=> qdriip_sys_clk_n,
		clk_ref_i				=> clk200,
		qdriip_cq_n				=> qdriip_cq_slv(0 downto 0),
		qdriip_q				=> qdriip_q(0),
		qdriip_k_p				=> qdriip_k_p_slv(0 downto 0),
		qdriip_k_n 				=> qdriip_k_n_slv(0 downto 0),
		qdriip_d				=> qdriip_d(0),
		qdriip_sa				=> qdriip_sa(0),
		qdriip_w_n				=> qdriip_w_n(0),
		qdriip_r_n 				=> qdriip_r_n(0),
		qdriip_bw_n				=> qdriip_bw_n(0),
		qdriip_dll_off_n 		=> qdriip_dll_off_n(0),
		qdriip_qvld				=> qdriip_qvld(0),
		app_wr_cmd0				=> sram_a_wr_cmd,
		app_wr_addr0			=> sram_a_wr_addr,
		app_wr_data0			=> sram_a_wr_data,
		app_wr_bw_n0			=> sram_a_wr_bw_n,
		app_rd_cmd0				=> sram_a_rd_cmd,
		app_rd_addr0			=> sram_a_rd_addr,
		app_rd_valid0			=> sram_a_rd_valid,
		app_rd_data0			=> sram_a_rd_data,
		app_wr_cmd1				=> '0',
		app_wr_addr1			=> (others => '0'),
		app_rd_cmd1				=> '0',
		app_rd_addr1			=> (others => '0'),
		app_wr_data1			=> (others => '0'),
		app_wr_bw_n1			=> (others => '0'),
		app_rd_valid1			=> open,
		app_rd_data1			=> open,
		clk						=> sram_clk,
		rst_clk					=> sram_a_rst_clk,
		init_calib_complete		=> sram_a_calib_complete,
		sys_rst					=> rst
	);
    
--****************************************************** SRAM-B INTERFACE ******************************************************

	-- Convert between standard logic and single bit standard logic vector types.
	qdriip_cq_slv(1) <= qdriip_cq(1);
	qdriip_k_p(1) <= qdriip_k_p_slv(1);
	qdriip_k_n(1) <= qdriip_k_n_slv(1);
	
	SRAM_B_MIG : entity work.qdr_sram_b
	port map (
		sys_rst_in				=> sys_rst_local,
		clocks_in				=> clocks_local,
		clk_ref_i				=> clk200,
		qdriip_cq_n				=> qdriip_cq_slv(1 downto 1),
		qdriip_q				=> qdriip_q(1),
		qdriip_k_p				=> qdriip_k_p_slv(1 downto 1),
		qdriip_k_n				=> qdriip_k_n_slv(1 downto 1),
		qdriip_d				=> qdriip_d(1),
		qdriip_sa				=> qdriip_sa(1),
		qdriip_w_n				=> qdriip_w_n(1),
		qdriip_r_n				=> qdriip_r_n(1),
		qdriip_bw_n				=> qdriip_bw_n(1),
		qdriip_dll_off_n		=> qdriip_dll_off_n(1),
		qdriip_qvld				=> qdriip_qvld(1),
		app_wr_cmd0				=> sram_b_wr_cmd,
		app_wr_addr0			=> sram_b_wr_addr,
		app_wr_data0			=> sram_b_wr_data,
		app_wr_bw_n0			=> sram_b_wr_bw_n,
		app_rd_cmd0				=> sram_b_rd_cmd,
		app_rd_addr0			=> sram_b_rd_addr,
		app_rd_valid0			=> sram_b_rd_valid,
		app_rd_data0			=> sram_b_rd_data,
		app_wr_cmd1				=> '0',
		app_wr_addr1			=> (others => '0'),
		app_wr_data1			=> (others => '0'),
		app_wr_bw_n1			=> (others => '0'),
		app_rd_cmd1				=> '0',
		app_rd_addr1			=> (others => '0'),
		app_rd_valid1			=> open,
		app_rd_data1			=> open,
		rst_clk					=> sram_b_rst_clk,
		init_calib_complete		=> sram_b_calib_complete
	);
	
end behavioral;
